﻿using TypographyContracts.BindingModels;
using TypographyContracts.BusinessLogicsContracts;
using TypographyContracts.SearchModels;
using TypographyDataModels.Models;
using Microsoft.Extensions.Logging;

namespace TypographyView
{
    public partial class FormPrinted : Form
    {
        private readonly ILogger _logger;
        private readonly IPrintedLogic _logic;
        private int? _id;
        private Dictionary<int, (IComponentModel, int)> _printedComponents;
        public int Id { set { _id = value; } }
        public FormPrinted(ILogger<FormPrinted> logger, IPrintedLogic logic)
        {
            InitializeComponent();
            _logger = logger;
            _logic = logic;
            _printedComponents = new Dictionary<int, (IComponentModel, int)>();
        }
        private void LoadData()
        {
            _logger.LogInformation("Загрузка компонент изделия");
            try
            {
                if (_printedComponents != null)
                {
                    dataGridViewComponents.Rows.Clear();
                    foreach (var pc in _printedComponents)
                    {
                        dataGridViewComponents.Rows.Add(new object[] { pc.Key,
                            pc.Value.Item1.ComponentName, pc.Value.Item2 });
                    }
                    textBoxPrice.Text = CalcPrice().ToString();
                }
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка загрузки компонент изделия");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        private double CalcPrice()
        {
            double price = 0;
            foreach (var elem in _printedComponents)
            {
                price += ((elem.Value.Item1?.Cost ?? 0) * elem.Value.Item2);
            }
            return Math.Round(price * 1.1, 2);
        }
        private void buttonRef_Click(object sender, EventArgs e)
        {
            LoadData();
        }
        private void buttonAdd_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(FormPrintedComponent));
            if (service is FormPrintedComponent form)
            {
                if (form.ShowDialog() == DialogResult.OK)
                {
                    if (form.ComponentModel == null)
                    {
                        return;
                    }
                    _logger.LogInformation("Добавление нового компонента: {ComponentName} - {Count}",
                        form.ComponentModel.ComponentName, form.Count);
                    if (_printedComponents.ContainsKey(form.Id))
                    {
                        _printedComponents[form.Id] = (form.ComponentModel, form.Count);
                    }
                    else
                    {
                        _printedComponents.Add(form.Id, (form.ComponentModel, form.Count));
                    }
                    LoadData();
                }
            }
        }
        private void buttonUpd_Click(object sender, EventArgs e)
        {
            if (dataGridViewComponents.SelectedRows.Count == 1)
            {
                var service = Program.ServiceProvider?.GetService(typeof(FormPrintedComponent));
                if (service is FormPrintedComponent form)
                {
                    int id = Convert.ToInt32(dataGridViewComponents.SelectedRows[0].Cells[0].Value);
                    form.Id = id;
                    form.Count = _printedComponents[id].Item2;
                    if (form.ShowDialog() == DialogResult.OK)
                    {
                        if (form.ComponentModel == null)
                        {
                            return;
                        }
                        _logger.LogInformation("Изменение компонента: {ComponentName} - {Count}",
                            form.ComponentModel.ComponentName, form.Count);
                        _printedComponents[form.Id] = (form.ComponentModel, form.Count);
                        LoadData();
                    }
                }
            }
        }
        private void buttonDel_Click(object sender, EventArgs e)
        {
            if (dataGridViewComponents.SelectedRows.Count == 1)
            {
                if (MessageBox.Show("Удалить запись?", "Вопрос", MessageBoxButtons.YesNo, MessageBoxIcon.Question) ==
                    DialogResult.Yes)
                {
                    try
                    {
                        _logger.LogInformation("Удаление компонента: { ComponentName} - { Count}",
                            dataGridViewComponents.SelectedRows[0].Cells[1].Value);
                        _printedComponents?.Remove(Convert.ToInt32(dataGridViewComponents.SelectedRows[0].Cells[0].Value));
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                    LoadData();
                }
            }
        }
        private void buttonSave_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxName.Text))
            {
                MessageBox.Show("Заполните название", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (string.IsNullOrEmpty(textBoxPrice.Text))
            {
                MessageBox.Show("Заполните цену", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (_printedComponents == null || _printedComponents.Count == 0)
            {
                MessageBox.Show("Заполните компоненты", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _logger.LogInformation("Сохранение изделия");
            try
            {
                var model = new PrintedBindingModel
                {
                    Id = _id ?? 0,
                    PrintedName = textBoxName.Text,
                    Price = Convert.ToDouble(textBoxPrice.Text),
                    PrintedComponents = _printedComponents
                };
                var operationResult = _id.HasValue ? _logic.Update(model) : _logic.Create(model);
                if (!operationResult)
                {
                    throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
                }
                MessageBox.Show("Сохранение прошло успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.OK;
                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка сохранения изделия");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        private void buttonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }

        private void FormPrinted_Load(object sender, EventArgs e)
        {
            if (_id.HasValue)
            {
                _logger.LogInformation("Загрузка изделия");
                try
                {
                    var view = _logic.ReadElement(new PrintedSearchModel { Id = _id.Value });
                    if (view != null)
                    {
                        textBoxName.Text = view.PrintedName;
                        textBoxPrice.Text = view.Price.ToString();
                        _printedComponents = view.PrintedComponents ?? new Dictionary<int, (IComponentModel, int)>();
                        LoadData();
                    }
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка загрузки изделия");
                    MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }
    }
}
