﻿using TypographyContracts.BindingModels;
using TypographyContracts.BusinessLogicsContracts;
using TypographyContracts.SearchModels;
using TypographyContracts.StoragesContracts;
using TypographyContracts.ViewModels;
using Microsoft.Extensions.Logging;
using TypographyDataModels.Enums;

namespace TypographyBusinessLogic.BusinessLogics
{
    public class OrderLogic : IOrderLogic
    {
        private readonly ILogger _logger;
        private readonly IOrderStorage _orderStorage;
        public OrderLogic(ILogger<OrderLogic> logger, IOrderStorage orderStorage)
        {
            _logger = logger;
            _orderStorage = orderStorage;
        }
        public List<OrderViewModel>? ReadList(OrderSearchModel? model)
        {
            _logger.LogInformation("ReadList. Id:{Id}", model?.Id);
            var list = model == null ? _orderStorage.GetFullList() : _orderStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count: {Count}", list.Count);
            return list;
        }
        public bool CreateOrder(OrderBindingModel model)
        {
            CheckModel(model);
            if (model.Status != OrderStatus.Неизвестен)
            {
                _logger.LogWarning("Order status change failed");
                return false;
            }
            model.Status = OrderStatus.Принят;

            if (_orderStorage.Insert(model) == null)
            {
                _logger.LogWarning("Order creation failed");
                return false;
            }
            return true;
        }
        public bool TakeOrderInWork(OrderBindingModel model)
        {
            return ChangeOrderStatus(model, OrderStatus.Выполняется);
        }
        public bool FinishOrder(OrderBindingModel model)
        {
            return ChangeOrderStatus(model, OrderStatus.Готов);
        }
        public bool DeliveryOrder(OrderBindingModel model)
        {
            return ChangeOrderStatus(model, OrderStatus.Выдан);
        }
        private void CheckModel(OrderBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (model.Count <= 0)
            {
                throw new ArgumentNullException("Кол-во печатных изделий в заказе должно быть больше 0", nameof(model.Count));
            }
            if (model.Sum <= 0)
            {
                throw new ArgumentNullException("Сумма заказа должна быть больше 0", nameof(model.Sum));
            }
            _logger.LogInformation("Order. Count: {Count}. Sum: {Sum} Id: {Id}", model.Count, model.Sum, model.Id);
        }
        private bool ChangeOrderStatus(OrderBindingModel model, OrderStatus newStatus)
        {
            CheckModel(model, false);
            var orderView = _orderStorage.GetElement(new OrderSearchModel { Id = model.Id });

            if (orderView == null || newStatus - orderView.Status != 1)
            {
                _logger.LogWarning("Order status change failed");
                return false;
            }
            model.Status = newStatus;
            if (newStatus == OrderStatus.Выдан)
            {
                model.DateImplement = DateTime.Now;
            }

            if (_orderStorage.Update(model) == null)
            {
                _logger.LogWarning("Order status change failed");
                return false;
            }
            return true;
        }
    }
}